#include "fx2.h"
#include "fx2regs.h"
#include "fx2sdly.h"            // SYNCDELAY macro

typedef unsigned char u8;
typedef signed char s8;
typedef unsigned short u16;
typedef signed short s16;
typedef unsigned long u32;
typedef signed long s32;

extern u8 ms_counter;

void jtag_tx(u8 n);
u8 jtag_trx(u8 n);
void jtag_wait_us(u8 us);

void tty_hex(u8 u);
void tty_out(s8 *s);


#define BSCMD_USER1			0x02
#define BSCMD_USER2			0x03
#define BSCMD_USRCODE		0x08	//32bit
#define BSCMD_IDCODE		0x09	//32bit
#define BSCMD_ISC_ENABLE	0x10
#define BSCMD_ISC_DISABLE	0x16
#define BSCMD_JPROGRAM		0x0b
#define BSCMD_ISC_DNA		0x31	//57bit
#define BSCMD_JSHUTDOWN		0x0d
#define BSCMD_CFG_IN		0x05
#define BSCMD_CFG_OUT		0x04
#define BSCMD_JSTART		0x0c
#define BSCMD_BYPASS		0x3f

u8 xdata jmem[64];
u8 xdata rmem[64];

void ptrset(){
	AUTOPTR1H=MSB( jmem );
	AUTOPTR1L=LSB( jmem );
	AUTOPTRH2=MSB( rmem );
	AUTOPTRL2=LSB( rmem );
}

void bscan_ir(u8 ir){
	jmem[0]=0x30;
	jmem[1]=0xff;
	jmem[2]=0x60;
	jmem[3]=ir | 0xc0;
	ptrset();
	jtag_tx(2);
}

void bscan_reset(){
	jmem[0]=0xff;
	jmem[1]=0xff;
	ptrset();
	jtag_tx(1);
}

void bscan_runtest(u8 n){
	u8 i;
	for(i=0; i<n; i++){
		jmem[i*2+0]=0x00;
		jmem[i*2+1]=0xff;
	}
	ptrset();
	jtag_tx(n);
}

void bscan_dr8r(u8 n){
	u8 i;

	jmem[0]=0x20;
	jmem[1]=0xff;
	ptrset();
	jtag_tx(1);

	for(i=0; i<n; i++){
		jmem[i*2+0]=(i == (n-1)) ? 0x80 : 0x00;
		jmem[i*2+1]=0xff;
	}
	ptrset();
	jtag_trx(n);

	jmem[0]=0x01;
	jmem[1]=0xff;
	ptrset();
	jtag_tx(1);
}

void bscan_dr8w_zero(u8 n){
	u8 i;

	jmem[0]=0x20;
	jmem[1]=0xff;

	for(i=0; i<n; i++){
		jmem[2+i*2+0]=(i == (n-1)) ? 0x80 : 0x00;
		jmem[2+i*2+1]=0x00;
	}

	i = 2+i*2;
	jmem[i++]=0x01;
	jmem[i++]=0xff;
	ptrset();
	jtag_tx(n+2);
}

void bscan_dr1wr_trx(){
	u8 bits;
	u8 i,j;

	bits = rmem[0];

	jmem[0]=0x20;
	jmem[1]=0xff;
	ptrset();
	jtag_tx(1);

	for(i = 0; i < ((bits+7)/8); i++){
		jmem[i*2+0] = 0x00;
		jmem[i*2+1] = rmem[i+1];
	}
	if((bits+7)/8 < (bits+11)/8){
		jmem[i*2+0] = 0x00;
		jmem[i*2+1] = 0x00;
		i++;
	}

	j = bits - 1;
	jmem[(j/8)*2] |= 1 << (j % 8);	//SHIFT-DR -> EXIT-DR
	j++;
	jmem[(j/8)*2] |= 1 << (j % 8);	//EXIT-DR -> UPDATE-DR

	ptrset();
	jtag_trx(i);
}

void bscan_r_mem(){
	u8 n;
	u8 i;


	jmem[0]=0x20;
	jmem[1]=0xff;
	jmem[2]=0x00;
	jmem[3]=rmem[0] << 1;
	jmem[4]=0x00;
	jmem[5]=(rmem[0] >> 7) + (rmem[1] << 1);
	ptrset();
	jtag_tx(3);

	n = rmem[2] * 2 - 1;
	for(i = 0; i < n ; i++){
		jmem[i*2+0] = 0x00;
		jmem[i*2+1] = 0x00;
	}
	ptrset();
	jtag_trx(n);

	jmem[0]=0x80;
	jmem[1]=0x00;
	jmem[2]=0x01;
	jmem[3]=0xff;
	AUTOPTR1H=MSB( jmem );
	AUTOPTR1L=LSB( jmem );
	jtag_trx(2);
}

void bscan_w_mem(){
	u8 n;
	u8 i;


	jmem[0]=0x20;
	jmem[1]=0xff;
	jmem[2]=0x00;
	jmem[3]=(rmem[0] << 1) + 1;	//Write
	jmem[4]=0x00;
	jmem[5]=(rmem[0] >> 7) + (rmem[1] << 1);
	ptrset();
	jtag_tx(3);

	n = rmem[2] * 2;
	for(i = 0; i < n ; i++){
		jmem[i*2+0] = 0x00;
		jmem[i*2+1] = rmem[3+i];
	}
	ptrset();
	jtag_tx(n);

	jmem[0]=0x03;
	jmem[1]=0x00;
	ptrset();
	jtag_tx(1);
}

void jtag_start()
{
	PORTACFG=0;
	OEA=0x83;		//PA7(TCK), PA1(TMS), PA0(TDI) is output
}

void jtag_end()
{
	OEA=0x00;		//all input
}

