//-----------------------------------------------------------------------------
//   File:      bulkloop.c
//   Contents:  Hooks required to implement USB peripheral function.
//
// $Archive: /USB/Examples/FX2LP/bulkloop/bulkloop.c $
// $Date: 3/23/05 2:55p $
// $Revision: 4 $
//
//
//-----------------------------------------------------------------------------
// Copyright 2003, Cypress Semiconductor Corporation
//-----------------------------------------------------------------------------
#pragma NOIV               // Do not generate interrupt vectors
//#define DEBUG
//#define FIX_USB_SERIAL

#include "fx2.h"
#include "fx2regs.h"
#include "syncdly.h"            // SYNCDELAY macro

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings

typedef unsigned char u8;
typedef signed char s8;
typedef unsigned short u16;
typedef signed short s16;
typedef unsigned long u32;
typedef signed long s32;

#define BSCMD_USER1			0x02
#define BSCMD_USER2			0x03
#define BSCMD_USRCODE		0x08	//32bit
#define BSCMD_IDCODE		0x09	//32bit
#define BSCMD_ISC_ENABLE	0x10
#define BSCMD_ISC_DISABLE	0x16
#define BSCMD_JPROGRAM		0x0b
#define BSCMD_ISC_DNA		0x31	//57bit
#define BSCMD_JSHUTDOWN		0x0d
#define BSCMD_CFG_IN		0x05
#define BSCMD_CFG_OUT		0x04
#define BSCMD_JSTART		0x0c
#define BSCMD_BYPASS		0x3f

void timer_init();
void jtag_tx(u8 n);
void jtag_tx_fix_tms(u8 n);
void jtag_trx(u8 n);
void jtag_wait_us(u8 us);
void jtag_tck100x(u8 n);
void jtag_start();
void jtag_end();
void bscan_ir(u8 ir);
void bscan_reset();
void bscan_runtest(u8 n);
void bscan_dr8r(u8 n);
void bscan_dr8r(u8 n);
void bscan_dr8w_zero(u8 n);
void bscan_dr1wr_trx();
void bscan_r_mem();
void bscan_w_mem();
void ptrset();
extern u8 xdata jmem[];
extern u8 xdata rmem[];
extern	u8 code StringDscr3;

u8 xdata cfg_mem[16];
u8 xdata rev[256];

bit fifo_width = 0;	//default 8bit FIFO
bit spa3_cfg = 0;
bit spa3_xc3s50a = 0;
//bit spa3_vita = 1;
u32 cfg_len;

#define VR_NAKALL_ON    0xD0
#define VR_NAKALL_OFF   0xD1

const s8 hexchar[16]={
'0','1','2','3','4','5','6','7',
'8','9','A','B','C','D','E','F'
};

#ifdef DEBUG
void tty_out(s8 *s){
	u8 i=0;
	while(s[i]){
		if(s[i]=='\n'){
			SBUF0=0x0d;
			for(;;) if(TI==1) break;
			TI=0;
			SBUF0=0x0a;
		}
		else{
			SBUF0=s[i];
		}
		i++;
		for(;;){
			if(TI==1) break;
		}
		TI=0;
	}
}

void tty_hex(u8 u){
	u8 s[3];
	s[0]=hexchar[u>>4];
	s[1]=hexchar[u & 0x0f];
	s[2]=0;
	tty_out(s);
}
#endif

//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//   The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------

void TD_Init(void)             // Called once at startup
{
	u8 i;
   // set the CPU clock to 48MHz and drive CLKOUT
   CPUCS = ((CPUCS & ~bmCLKSPD) | bmCLKSPD1 | bmCLKOE) ;

	//set FD bus 8bit
	EP2FIFOCFG = 0x04;
	SYNCDELAY;
	EP4FIFOCFG = 0x04;
	SYNCDELAY;
	EP6FIFOCFG = 0x04;
	SYNCDELAY;
	EP8FIFOCFG = 0x04;
	SYNCDELAY;
	
/*
	//check vita board
	for(j=0;j<255;j++){
		i = IOA;
		i = i & 0x74;
		if(i != 0x44) spa3_vita = 0;
	}
*/

#ifdef DEBUG
	//serial port init
	UART230=0x01;	//port0 set 115K baud
	SCON0    = 0x50; // 0101 0000 mode=1, REN_1=1
	TI       = 0;

	tty_out("\n*** SPA3 by OPTIMIZE ***");
	tty_hex(0);
 #endif
	
	timer_init();

	EP1OUTCFG = 0xA0;
	EP1INCFG = 0xA0;
	SYNCDELAY;                    // see TRM section 15.14
	EP1OUTBC=0;


	REVCTL = 0x03; SYNCDELAY;		// see TRM 15.5.9

	EP2CFG = 0xA0;	//(OUT, Size = 512, buf = Quad (Buf x4), BULK);
	SYNCDELAY;                    

	EP4CFG = 0x00;
	SYNCDELAY;                    

	EP6CFG = 0xE0;	//(IN, Size = 512, buf = Quad (Buf x4), BULK) 
	SYNCDELAY;

	EP8CFG = 0x00;
	SYNCDELAY;                    

	FIFOPINPOLAR = 0x00; SYNCDELAY;
	EP6AUTOINLENH = 0x02; SYNCDELAY;
	EP6AUTOINLENL = 0x00; SYNCDELAY;

	FIFORESET = 0x80; SYNCDELAY; // activate NAK-ALL to avoid race conditions
	FIFORESET = 0x02; SYNCDELAY; // reset, FIFO 2
	FIFORESET = 0x04; SYNCDELAY; // reset, FIFO 4
	FIFORESET = 0x06; SYNCDELAY; // reset, FIFO 6
	FIFORESET = 0x08; SYNCDELAY; // reset, FIFO 8
	FIFORESET = 0x00; SYNCDELAY; // deactivate NAK-ALL

	// enable dual autopointer feature & inc
	AUTOPTRSETUP = 0x07;

	IOD = 0x00;	//MODE_IDLE
	OED = 0xc0;	//MODE is output

	PINFLAGSAB = 0xe8;	//FLAGA=ep2EF FLAGB=ep6FF
	SYNCDELAY;


	for(i = 0;;){
		if(i < 16) cfg_mem[i] = 0;
		rev[i] = (i<<7)+((i<<5)&0x40)+((i<<3)&0x20)+((i<<1)&0x10)+
			((i>>1)&0x08)+((i>>3)&0x04)+((i>>5)&0x02)+(i>>7);
		i++;
		if(i == 0) break;
	}

	jtag_start();
	bscan_reset();
	bscan_runtest(2);

	bscan_ir(BSCMD_IDCODE);
	bscan_dr8r(4);
	bscan_ir(BSCMD_IDCODE);
	bscan_dr8r(4);
	for(i=0; i<4; i++)	cfg_mem[i] = rmem[i];

	if((rmem[3]==0x02)&&(rmem[2]==0x21)&&(rmem[1]==0x00)&&(rmem[0]==0x93)){	//XC3S50A
		spa3_xc3s50a = 1;
		bscan_ir(BSCMD_ISC_ENABLE);
		bscan_ir(BSCMD_JPROGRAM);
		bscan_runtest(2);
		bscan_ir(BSCMD_ISC_DNA);
		bscan_dr8r(8);
		bscan_runtest(2);
		for(i = 0; i< 8; i++){
			jmem[7-i] = rev[rmem[i]];
		}
		for(i = 0; i< 7; i++){
			rmem[i] = (jmem[i] >> 7) + (jmem[i+1] << 1);
		}
		rmem[i] = (jmem[i] >> 7);
		AUTOPTR1H=MSB( &StringDscr3 + 2);
		AUTOPTR1L=LSB( &StringDscr3 + 2);
		for(i=0;i<8;i++){
			//tty_hex(rmem[7-i]);
#ifdef FIX_USB_SERIAL
			XAUTODAT1 = '0';
			jmem[i] = XAUTODAT1;
			XAUTODAT1 = '1';
			jmem[i] = XAUTODAT1;
#else
			XAUTODAT1 = hexchar[rmem[7-i] >> 4];
			jmem[i] = XAUTODAT1;
			XAUTODAT1 = hexchar[rmem[7-i] & 0x0f];
			jmem[i] = XAUTODAT1;
#endif
			cfg_mem[8+i] = rmem[i];
//			if(spa3_vita) cfg_mem[8+i]=0;
		}
	}
	jtag_end();
}

extern u8 ms_counter;

#define CMD_EP6IN_START		0x50	//
#define	CMD_EP6IN_STOP		0x51	//
#define	CMD_EP2OUT_START	0x52	//
#define	CMD_EP2OUT_STOP		0x53	//
#define CMD_PORTD_CFG		0x56	//dir(OED)
#define	CMD_PORTD_READ		0x57	//return 1byte
#define	CMD_PORTD_WRITE		0x58	//value
#define	CMD_IFCONFIG		0x59	//value
#define CMD_SET_FIFO_8		0x5a	//
#define CMD_SET_FIFO_16		0x5b	//

#define	CMD_JTAG_TX			0x60	//len,tms0,tdi0,tms1,tdi1,.....
#define	CMD_JTAG_TRX		0x61	//len,tms0,tdi0,tms1,tdi1,..... return len
#define CMD_JTAG_WAIT_US	0x62	//us
#define CMD_JTAG_WAIT_MS	0x63	//ms
#define CMD_JTAG_START		0x64
#define CMD_JTAG_END		0x65
#define CMD_JTAG_TCK100X	0x66	//len  tck 100*len cycle
#define	CMD_JTAG_TX_FIX_TMS	0x67	//len, tdi0,tdi1,.....

#define CMD_CFG_READ		0x70	//return 16byte
#define CMD_CFG_WRITE		0x71	//cfg[0],....[15]
#define CMD_SPA3_CFG		0x72	//len(4byte)
#define CMD_SPA3_PORT		0x73	//bits, data0,...
#define CMD_SPA3_R_MEM		0x74	//addr_l, addr_h, words(max 16)   return words*2 bytes
#define CMD_SPA3_W_MEM		0x75	//addr_l, addr_h, words(max 16), data0, data1, ....

void TD_Poll(void)              // Called repeatedly while the device is idle
{
	u8 i,j;
	u8 cmd_cnt,ret_cnt;
	if(!(EP1OUTCS & bmEPBUSY))
	{
		cmd_cnt=EP1OUTBC;
		AUTOPTR1H=MSB( &EP1OUTBUF );
        AUTOPTR1L=LSB( &EP1OUTBUF );
		AUTOPTRH2=MSB( &EP1INBUF );
        AUTOPTRL2=LSB( &EP1INBUF );
		ret_cnt=0;
		if(spa3_cfg){
			if(cfg_len == (u32)cmd_cnt){
				spa3_cfg = 0;
				jtag_tx_fix_tms(cmd_cnt-1);
				jmem[0]=0x80;
				jmem[1]=rev[XAUTODAT1];
				jmem[2]=0x01;
				jmem[3]=0xff;
				ptrset();
				jtag_tx(2);

				bscan_ir(BSCMD_ISC_DISABLE);
				bscan_runtest(2);
				bscan_ir(BSCMD_BYPASS);
				bscan_dr8w_zero(1);

				bscan_ir(BSCMD_JSTART);
				bscan_runtest(2);
				bscan_ir(BSCMD_BYPASS);
				bscan_ir(BSCMD_BYPASS);
				
				jtag_end();
			}
			else{
				jtag_tx_fix_tms(cmd_cnt);
			}
			cfg_len -= cmd_cnt;
		}
		else{
			for(;;){
				if(cmd_cnt-- == 0) break;
				switch(i = XAUTODAT1){
				case CMD_CFG_READ:
					for(i=0; i<16; i++){
						XAUTODAT2=cfg_mem[i];
					}
					ret_cnt+=16;
					break;
				case CMD_CFG_WRITE:
					for(i=0; i<16; i++){
						cfg_mem[i]=XAUTODAT1;
					}
					cmd_cnt-=16;
					break;
				case CMD_SPA3_CFG:
					if(!spa3_xc3s50a) break;
//					if(spa3_vita) break;
					spa3_cfg = 1;
					*((u8 *)&cfg_len+3) = XAUTODAT1;	//big endian
					*((u8 *)&cfg_len+2) = XAUTODAT1;
					*((u8 *)&cfg_len+1) = XAUTODAT1;
					*((u8 *)&cfg_len+0) = XAUTODAT1;
					cmd_cnt-=4;
					
					jtag_start();
					bscan_runtest(2);
					bscan_ir(BSCMD_JPROGRAM);
					bscan_ir(BSCMD_CFG_IN);
					jtag_tck100x(100);
					bscan_ir(BSCMD_CFG_IN);
					bscan_dr8w_zero(2);
					bscan_ir(BSCMD_CFG_IN);

					jmem[0]=0x20;	//ready for bit data
					jmem[1]=0xff;
					ptrset();
					jtag_tx(1);
					break;
				case CMD_SPA3_PORT:
					for(i = 0; i < cmd_cnt; i++){
						rmem[i] = XAUTODAT1;
					}
					jtag_start();
					bscan_runtest(2);
					bscan_ir(BSCMD_USER1);
					i = (rmem[0] + 7) / 8;
					bscan_dr1wr_trx();
					cmd_cnt = 0;	//This command must be final comand. Because use autoptr in bscan_dr1wr_trx().
					AUTOPTRH2=MSB( &EP1INBUF );
					AUTOPTRL2=LSB( &EP1INBUF );
					for(j=0; j<i; j++){
						XAUTODAT2 = rmem[j];
						ret_cnt++;
					}
					jtag_end();
					break;
				case CMD_SPA3_R_MEM:
					for(i = 0; i < cmd_cnt; i++){
						rmem[i] = XAUTODAT1;
					}
					jtag_start();
					bscan_runtest(2);
					bscan_ir(BSCMD_USER2);
					i = rmem[2]*2;
					bscan_r_mem();
					cmd_cnt = 0;	//This command must be final comand. Because use autoptr in bscan_r_mem().
					AUTOPTRH2=MSB( &EP1INBUF );
					AUTOPTRL2=LSB( &EP1INBUF );
					for(j=0; j<i; j++){
						XAUTODAT2 = rmem[j];
						ret_cnt++;
					}
					jtag_end();
					break;
				case CMD_SPA3_W_MEM:
					for(i = 0; i < cmd_cnt; i++){
						rmem[i] = XAUTODAT1;
					}
					jtag_start();
					bscan_runtest(2);
					bscan_ir(BSCMD_USER2);
					bscan_w_mem();
					cmd_cnt = 0;	//This command must be final comand. Because use autoptr in bscan_w_mem().
					jtag_end();
					break;
				case CMD_JTAG_TX:
					i=XAUTODAT1;
					jtag_tx(i);
					cmd_cnt-=(i+i+1);
					break;
				case CMD_JTAG_TX_FIX_TMS:
					i=XAUTODAT1;
					jtag_tx_fix_tms(i);
					cmd_cnt-=(i+1);
					break;
				case CMD_JTAG_TRX:
					i=XAUTODAT1;
					jtag_trx(i);
					ret_cnt+=(i);
					cmd_cnt-=(i+i+1);
					break;
				case CMD_JTAG_WAIT_US:
					jtag_wait_us(XAUTODAT1);
					cmd_cnt--;
					break;
				case CMD_JTAG_TCK100X:
					jtag_tck100x(XAUTODAT1);
					cmd_cnt--;
					break;
				case CMD_JTAG_WAIT_MS:
					ms_counter=XAUTODAT1;
					cmd_cnt--;
					break;
				case CMD_JTAG_START:
					jtag_start();
					break;
				case CMD_JTAG_END:
					jtag_end();
					break;
				case CMD_PORTD_CFG:
					OED = XAUTODAT1;
 				    cmd_cnt-=1;
					break;
				case CMD_PORTD_WRITE:
					IOD = XAUTODAT1;
					cmd_cnt-=1;
					break;
				case CMD_PORTD_READ:
					XAUTODAT2 = IOD;
					ret_cnt++;
					break;
				case CMD_IFCONFIG:
					IFCONFIG = XAUTODAT1;
					cmd_cnt-=1;
					break;
				case CMD_EP6IN_START:
					FIFORESET = 0x06;	// reset, FIFO 6
					SYNCDELAY;

					EP6FIFOCFG = 0x0C | fifo_width;	//Auto-IN
					SYNCDELAY;
					break;
				case CMD_EP6IN_STOP:
					EP6FIFOCFG = 0x04 | fifo_width;
					break;
				case CMD_EP2OUT_START:
					FIFORESET = 0x02;	// reset, FIFO 2
					SYNCDELAY;

					OUTPKTEND = 0x82;
					SYNCDELAY;
					OUTPKTEND = 0x82;
					SYNCDELAY;
					OUTPKTEND = 0x82;
					SYNCDELAY;
					OUTPKTEND = 0x82;
					SYNCDELAY;
					EP2FIFOCFG = 0x10 | fifo_width;	//Auto-OUT
					SYNCDELAY;
					break;
				case CMD_EP2OUT_STOP:
					EP2FIFOCFG = 0x04 | fifo_width;
					break;
				case CMD_SET_FIFO_8:
				case CMD_SET_FIFO_16:
					fifo_width = i - CMD_SET_FIFO_8;
					break;
				}
			}
		}
		EP1OUTBC=0;	//arm EP1OUT
		if(ret_cnt)
		{
			EP1INBC=ret_cnt;
		}
	}
}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   // Called when a Set Configuration command is received
{
   Configuration = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   // Called when a Get Configuration command is received
{
   EP0BUF[0] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       // Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[2];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       // Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
  BYTE tmp;
  
  switch (SETUPDAT[1])
  {
     case VR_NAKALL_ON:
        tmp = FIFORESET;
        tmp |= bmNAKALL;      
        SYNCDELAY;                    
        FIFORESET = tmp;
        break;
     case VR_NAKALL_OFF:
        tmp = FIFORESET;
        tmp &= ~bmNAKALL;      
        SYNCDELAY;                    
        FIFORESET = tmp;
        break;
     default:
        return(TRUE);
  }

  return(FALSE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
      ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}
