#include "StdAfx.h"
#include "spa3.h"

#define CMD_EP6IN_START		0x50	//
#define	CMD_EP6IN_STOP		0x51	//
#define	CMD_EP2OUT_START	0x52	//
#define	CMD_EP2OUT_STOP		0x53	//
#define CMD_PORTD_CFG		0x56	//dir(OED)
#define	CMD_PORTD_READ		0x57	//return 1byte
#define	CMD_PORTD_WRITE		0x58	//value
#define	CMD_IFCONFIG		0x59	//value
#define CMD_SET_FIFO_8		0x5a	//
#define CMD_SET_FIFO_16		0x5b	//

#define CMD_CFG_READ		0x70	//return 16byte
#define CMD_CFG_WRITE		0x71	//cfg[0],....[15]
#define CMD_SPA3_CFG		0x72	//len(4byte)
#define CMD_SPA3_PORT		0x73	//bits, data0,...
#define CMD_SPA3_R_MEM		0x74	//addr_l, addr_h, words(max 16)   return words*2 bytes
#define CMD_SPA3_W_MEM		0x75	//addr_l, addr_h, words(max 16), data0, data1, ....

#define	MODE_IDLE	0
#define	MODE_START	1

#define DIR_FX2PC	0	//EP6IN (FX2 -> PC) transfer
#define DIR_PC2FX	1	//EP2OUT(PC -> FX2) transfer

//#define DEBUG_FW

u8 fw_inc[] =
//#include "fw_fix_usb_serial.inc"
#include "fw.inc"

spa3::spa3(void)
{
	olen = ilen = 0;
#ifdef DEBUG_FW
	FILE *fp_fw;
	fopen_s(&fp_fw, "fw\\fifo.iic","rb");
	fw_iic = new u8[1024*8];
	fread(fw_iic, 1, 1024*8, fp_fw);
	fclose(fp_fw);
#else
	fw_iic = fw_inc;
#endif
}

spa3::~spa3(void)
{
#ifdef DEBUG_FW
	delete fw_iic;
#endif
}

void spa3::ep6in_start()
{
	odata[olen++]=CMD_EP6IN_START;
	usb_trx();
}

void spa3::ep6in_stop()
{
	odata[olen++]=CMD_EP6IN_STOP;
	usb_trx();
}

void spa3::ep2out_start()
{
	odata[olen++]=CMD_EP2OUT_START;
	usb_trx();
}

void spa3::ep2out_stop()
{
	odata[olen++]=CMD_EP2OUT_STOP;
	usb_trx();
}

void spa3::portd_cfg(u8 dir)
{
	odata[olen++]=CMD_PORTD_CFG;
	odata[olen++]=dir;
	usb_trx();
}

u8   spa3::portd_read()
{
	odata[olen++]=CMD_PORTD_READ;
	ilen++;
	usb_trx();
	return idata[0];
}

void spa3::portd_write(u8 val)
{
	odata[olen++]=CMD_PORTD_WRITE;
	odata[olen++]=val;
	usb_trx();
}

void spa3::ifconfig(u8 val)
{
	odata[olen++]=CMD_IFCONFIG;
	odata[olen++]=val;
	usb_trx();
}

void spa3::set_fifo_8()
{
	odata[olen++]=CMD_SET_FIFO_8;
	usb_trx();
}

void spa3::set_fifo_16()
{
	odata[olen++]=CMD_SET_FIFO_16;
	usb_trx();
}


void spa3::usb_trx()
{
	bool bResult = true;
	if (olen == 0) return;
	bResult  = usb->xfer(outep, odata, olen);
	if (bResult && (ilen != 0))
	{
		usb->xfer(inep, idata, ilen);
	}
	olen = ilen = 0;
}

bool spa3::init(cusb2 *_usb, u64 *dev_dna, u8 *bit_buf){
	s32 i,j;
	u32 bit_sec;
	u32 bit_len;
	u8 *buf;
	struct{
		u32 idcode;
		u32 bit_sec_cfged;	//SPA3ɏ܂Ăbitt@C̃^CX^v
		u64 dev_dna;
	} cfg_mem;

	usb = _usb;

	if(usb->fwload(dev_dna, fw_iic) != true) return false;	//SPA3 FW[h

	inep = usb->get_endpoint(0x81);
	outep = usb->get_endpoint(0x01);

	//XILINX bitt@C^CX^vCFGC[W擾
	i = 13;
	for(j = 0; j < 6; j++){
		if(bit_buf[i] == 0x64){
			bit_sec = ((bit_buf[i+10]-'0')+10*(bit_buf[i+9]-'0')+
				60*((bit_buf[i+7]-'0')+10*(bit_buf[i+6]-'0'))+60*60*((bit_buf[i+4]-'0')+10*(bit_buf[i+3]-'0')));
		}
		if(bit_buf[i] == 0x65){
			bit_len = (bit_buf[i+1] << 24) + (bit_buf[i+2] << 16) + (bit_buf[i+3] << 8)+bit_buf[i+4];
			buf = bit_buf + i + 5;
			break;
		}
		i += 3 + (bit_buf[i+1] << 8) + bit_buf[i+2];
	}
	if(j == 6) return false;

	//FX2̏ԃ(cfg_mem)ǂݍ
	odata[olen++]=CMD_CFG_READ;
	ilen += 16;
	usb_trx();
	memcpy(&cfg_mem, idata, sizeof(cfg_mem));
	*dev_dna = cfg_mem.dev_dna;
	if(cfg_mem.dev_dna == 0)
		return false;	//vita{[ĥߋI

	if(cfg_mem.bit_sec_cfged != bit_sec){	//ݍς݂ƈقȂꍇ̂݃RtBO
		LONG l;
		odata[olen++]=CMD_SPA3_CFG;
		odata[olen++]=(u8)bit_len;
		odata[olen++]=(u8)(bit_len>>8);
		odata[olen++]=(u8)(bit_len>>16);
		odata[olen++]=(u8)(bit_len>>24);
		usb_trx();
		l = bit_len;
		outep->XferData(buf, l);

		//FX2̏ԃ(cfg_mem)ɏ
		odata[olen++]=CMD_CFG_WRITE;
		cfg_mem.bit_sec_cfged = bit_sec;
		memcpy(odata+olen, &cfg_mem, sizeof(cfg_mem));
		olen += 16;
		usb_trx();
	}

	return true;
}

void spa3::port(u8 bits, u8 *out, u8 *in){
	s32 i;
	odata[olen++]=CMD_SPA3_PORT;
	odata[olen++]=(u8)bits;
	for(i = 0; i < (bits+7)/8; i++){
		odata[olen++]=out[i];
		ilen++;
	}
	usb_trx();
	memcpy(in, idata, (bits+7)/8);
}

void spa3::mem_read(u16 addr, u16 word, u16 *buf){
	s32 i, l;

	for(i = 0; i < word; ){
		odata[olen++]=CMD_SPA3_R_MEM;
		odata[olen++]=(u8)(addr + i);
		odata[olen++]=(u8)((addr + i)>>8);
		if(word - i > 16){
			l = 16;
		}
		else{
			l = word - i;
		}
		ilen = l*2;
		odata[olen++]=l;
		usb_trx();
		memcpy(buf+i, idata, l*2);
		i += l;
	}
}

void spa3::mem_write(u16 addr, u16 word, u16 *buf){
	s32 i, j, l;

	for(i = 0; i < word; ){
		odata[olen++]=CMD_SPA3_W_MEM;
		odata[olen++]=(u8)(addr + i);
		odata[olen++]=(u8)((addr + i)>>8);
		if(word - i > 16){
			l = 16;
		}
		else{
			l = word - i;
		}
		odata[olen++]=l;
		for(j = 0; j < l; j++){
			odata[olen++]=*((u8 *)(buf + i + j) + 0);
			odata[olen++]=*((u8 *)(buf + i + j) + 1);
		}
		usb_trx();
		i += l;
	}
}
