`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:11:55 06/25/2012 
// Design Name: 
// Module Name:    jtag_port 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module jtag_port(OUT_PORT, IN_PORT, CLK, WE, ADDR, DIN, DOUT);
	parameter W = 8;
   output [W-1:0] OUT_PORT;
   input  [W-1:0] IN_PORT;

	output CLK;
	output WE;
	output [12:0] ADDR;
	output [15:0] DOUT;
	input [15:0] DIN;
	
	reg [W-1:0] sreg;
	reg [W-1:0] oreg;
	assign OUT_PORT = oreg;
	assign tdo = sreg[0];
	
	wire capture;
	wire drclk;
	wire sel1;
	wire shift;
	wire tck;
	wire tdi;
	wire update;
	
	wire drclk2;
	wire sel2;
	wire tdo2;
		
	BSCAN_SPARTAN3A jtag (
		.CAPTURE(capture), 	// CAPTURE output from TAP controller
		.DRCK1(drclk), 		// Data register output for USER1 functions
		.DRCK2(drclk2), 		// Data register output for USER2 functions
		.RESET(), 				// Reset output from TAP controller
		.SEL1(sel1), 			// USER1 active output
		.SEL2(sel2), 			// USER2 active output
		.SHIFT(shift), 		// SHIFT output from TAP controller
		.TCK(tck), 				// TCK output from TAP controller
		.TDI(tdi), 				// TDI output from TAP controller
		.TMS(), 					// TMS output from TAP controller
		.UPDATE(update), 		// UPDATE output from TAP controller
		.TDO1(tdo),				// Data input for USER1 function
		.TDO2(tdo2) 			// Data input for USER2 function
	);
	
	always @(posedge drclk)
	begin
		if(capture)
			sreg <= IN_PORT;
		else if(shift)
			sreg <= {tdi,sreg[W-1:1]};
	end
	
	always @(posedge tck)
	begin
		if(update && sel1)
			oreg <= sreg;
	end
	
	reg rw_mode;
	reg [12:0] raddr;
	reg [15:0] rdata;
	reg ad_mode;
	reg [3:0] cnt;
	reg first;
	reg reg_w;
	
	assign CLK = tck;
	assign WE = reg_w;
	assign ADDR = raddr;
	assign DOUT = rdata;
	assign tdo2 = rdata[0];

	always @(posedge drclk2 or posedge capture)
	begin
		if(capture) begin
			rw_mode <= 0;
			ad_mode <= 0;
			raddr <= 0;
			rdata <= 0;
			cnt <= 0;
			first <= 1;
			reg_w <= 0;
		end else begin
			if(shift) begin
				if(!ad_mode) begin
					if(!rw_mode) begin
						//read
						if(cnt == 14) begin
							ad_mode <= 1;
							cnt <= 0;
						end else begin
							{raddr, rw_mode} <= {tdi, raddr};
							cnt <= cnt + 1;
						end
					end else begin
						//write
						if(cnt == 15) begin
							ad_mode <= 1;
							cnt <= 0;
						end else if(cnt == 14) begin
							cnt <= cnt + 1;
						end else begin
							{raddr, rw_mode} <= {tdi, raddr};
							cnt <= cnt + 1;
						end
					end
				end else begin
					if(!rw_mode) begin
						//read
						if(cnt == 0) begin
							rdata <= DIN;
							raddr <= raddr + 1;
						end else begin
							rdata <= {tdi, rdata[15:1]};
						end
					end else begin
						//write
						if((cnt == 0) && !first) begin
							raddr <= raddr + 1;
						end else begin
							first <= 0;
						end
						if(cnt == 15)
							reg_w <= 1;
						else
							reg_w <= 0;
						rdata <= {tdi, rdata[15:1]};
					end
					cnt <= cnt + 1;
				end
			end
		end
	end
	
endmodule
