//`define DEBUG
module spd_chk(FX2_SLOE, FX2_SLRD, FX2_SLWR, FX2_FD, FX2_FIFOADR, FX2_PKTEND,
		FX2_FLAGA, FX2_FLAGB, FX2_FLAGC, FX2_IFCLK, CLK12
`ifdef DEBUG
		,TP
`endif
		);

output FX2_SLOE;
output FX2_SLRD;
output FX2_SLWR;
inout  [15:0] FX2_FD;
output [1:0] FX2_FIFOADR;
output FX2_PKTEND;
input  FX2_FLAGA;
input  FX2_FLAGB;
input  FX2_FLAGC;
input  FX2_IFCLK;

input  CLK12;

`ifdef DEBUG
output [11:0] TP;
assign TP[0] = FX2_SLRD;
assign TP[1] = FX2_SLWR;
assign TP[2] = FX2_FLAGA;
assign TP[3] = FX2_FLAGB;
assign TP[11:4] = FX2_FD;
`endif

`define MODE_IDLE	0
`define	MODE_START	1
wire  _MODE;
reg	MODE;

wire  RST;

`define DIR_FX2PC	0
`define DIR_PC2FX	1
wire  DIR;
	
wire [31:0] init_val;

wire [35:0] in_port;
wire [35:0] out_port;


wire [9:0] ram_addr;
wire ram_clk;
wire ram_we;
wire [15:0] ram_din;
wire [15:0] ram_dout;

wire [13:0] jram_addr;
wire jram_clk;
wire jram_we;
wire [15:0] jram_din;
wire [15:0] jram_dout;

jtag_port #(35) jtag_port(.OUT_PORT(out_port), .IN_PORT(in_port),
 .CLK(jram_clk), .WE(jram_we), .ADDR(jram_addr), .DIN(jram_dout), .DOUT(jram_din));

ram ram (
  .clka(ram_clk), // input clka
  .wea(ram_we), // input [0 : 0] wea
  .addra(ram_addr), // input [9 : 0] addra
  .dina(ram_din), // input [15 : 0] dina
  .douta(ram_dout), // output [15 : 0] douta
  .clkb(jram_clk), // input clkb
  .web(jram_we), // input [0 : 0] web
  .addrb(jram_addr[9:0]), // input [9 : 0] addrb
  .dinb(jram_din), // input [15 : 0] dinb
  .doutb(jram_dout) // output [15 : 0] doutb
);

reg [15:0] addr;
assign ram_addr = addr[9:0];
assign ram_we = 1;
assign ram_clk = FX2_IFCLK;

reg slwr,slrd;
reg [31:0] cnt32;
reg [30:0] wait_cnt31;
reg cnt2;
reg [15:0] cnt32_16;
reg cmp_err;
reg first_skip;	//first data is invalid

reg [15:0] wait16;

always @(posedge FX2_IFCLK)
begin
	MODE <= _MODE;
end

always @(posedge FX2_IFCLK)
begin
	if(RST) begin
		cnt32 <= init_val;
		wait_cnt31 <= 0;
		cnt2 <= 0;
		slwr <= 1;
		slrd <= 1;
		cmp_err <= 0;
		first_skip <= 1;
		wait16 <= 0;
		addr <= 0;
	end else if((((FX2_FLAGB == 1)&&(DIR == `DIR_FX2PC))||((FX2_FLAGA == 1)&&(DIR == `DIR_PC2FX)))
	 && (MODE == `MODE_START)) begin
		if(DIR == `DIR_FX2PC) begin
			slwr <= 0;
			if     (cnt2 == 0) cnt32_16 <= cnt32[15:0];
			else if(cnt2 == 1) cnt32_16 <= cnt32[31:16];
			if(cnt2 == 1) cnt32 <= cnt32 + 1;
		end
		if(DIR == `DIR_PC2FX) begin
			slrd <= 0;
			if((cnt2 == 1) && (FX2_FD != cnt32[15: 0])) cmp_err <= 1;
			if(first_skip == 0) begin
				if((cnt2 == 0) && (FX2_FD != cnt32[31:16])) cmp_err <= 1;
				if(cnt2 == 0) cnt32 <= cnt32 + 1;
			end
			first_skip <= 0;
		end
		cnt2 <= cnt2 + 1;
		if(wait16 >= 2) begin
			addr <= addr + 1;
		end
		wait16 <= 0;
	end else begin
		if(MODE == `MODE_START) begin
			wait_cnt31 <= wait_cnt31 + 1;
			wait16 <= wait16 + 1;
		end
		slwr <= 1;
		slrd <= 1;
		cnt2 <= 0;
		first_skip <= 1;
	end
end


assign {RST, _MODE, DIR, init_val[31:0]} = out_port;
assign in_port = {cmp_err, wait_cnt31};
assign ram_din = wait16;

assign FX2_SLOE = ((MODE == `MODE_START)&&(DIR == `DIR_PC2FX)) ? 0 : 1;
assign FX2_FIFOADR = (DIR == `DIR_FX2PC) ? 2 : 0;
assign FX2_PKTEND = 1;
assign FX2_SLRD = slrd;
assign FX2_SLWR = slwr;
assign FX2_FD = ((MODE == `MODE_START) && (DIR == `DIR_PC2FX)) ? 16'bzzzzzzzzzzzzzzzz : cnt32_16;

endmodule
